#!/bin/sh

# TEST: Branches: b0
# TEST:  add b1 before b0
# TEST:  add b1 after b0

source scaffold

function files {
cat <<FILES
d /n/lower
d /n/lower/b0
d /n/lower/b1
d /n/lower/b2
d /n/lower/b3
d /n/lower/b4
FILES
}

function beforefiles {
cat <<FILES
f /n/lower/b0/b0-only
f /n/lower/b0/a
f /n/lower/b1/a
f /n/lower/b2/a
f /n/lower/b3/a
f /n/lower/b4/a
FILES
}

function afterfiles {
	beforefiles
}

( files ; beforefiles) | create_hierarchy

mount_union "" /n/lower/b0

checktype $MOUNTPOINT/a 'f'
echo "/n/lower/b0/a" |diff /mnt/unionfs/a -
unionctl /mnt/unionfs --add --before /n/lower/b0 /n/lower/b1
echo "/n/lower/b1/a" |diff /mnt/unionfs/a -

unmount_union

echo -n "[add before] "

( files ; afterfiles )  | check_hierarchy /n/lower

( files ; beforefiles) | create_hierarchy

mount_union "" /n/lower/b0

checktype $MOUNTPOINT/a 'f'
echo "/n/lower/b0/a" |diff /mnt/unionfs/a -
unionctl /mnt/unionfs --add --after /n/lower/b0 /n/lower/b1
echo "/n/lower/b0/a" |diff /mnt/unionfs/a -

unmount_union

echo -n "[add after] "

( files ; afterfiles )  | check_hierarchy /n/lower

( files ; beforefiles) | create_hierarchy

mount_union "" /n/lower/b0

checktype $MOUNTPOINT/a 'f'
echo "/n/lower/b0/a" |diff /mnt/unionfs/a -
for ((i = 1; i <= 4; i++)) ; do
	unionctl /mnt/unionfs --add /n/lower/b$i
	echo "/n/lower/b$i/a" |diff /mnt/unionfs/a -
done

unmount_union

( files ; afterfiles )  | check_hierarchy /n/lower

echo -n "[add multiple] "

( files ; beforefiles) | create_hierarchy

mount_union "" /n/lower/b1 /n/lower/b0

echo "/n/lower/b1/a" |diff /mnt/unionfs/a -
unionctl /mnt/unionfs/b0-only --remove /n/lower/b1
echo "/n/lower/b0/a" |diff /mnt/unionfs/a -

unmount_union

( files ; afterfiles )  | check_hierarchy /n/lower

echo -n "[remove] "

( files ; beforefiles) | create_hierarchy

mount_union "" /n/lower/b0

echo "/n/lower/b0/a" |diff /mnt/unionfs/a -
for ((i = 1; i <= 4; i++)) ; do
	unionctl /mnt/unionfs/b0-only --add /n/lower/b$i
	echo "/n/lower/b$i/a" |diff /mnt/unionfs/a -
done

for ((i = 1; i < 4; i++)) ; do
	unionctl /mnt/unionfs/b0-only --remove /n/lower/b$i
	echo "/n/lower/b4/a" |diff /mnt/unionfs/a -
done
unionctl /mnt/unionfs/b0-only --remove /n/lower/b4
echo "/n/lower/b0/a" |diff /mnt/unionfs/a -

unmount_union

( files ; afterfiles )  | check_hierarchy /n/lower

echo -n "[remove multiple] "

echo "OK"
exit 0
